/*************************************************************************
 *
 *
 *    (c) Copyright Olimex 2010
 *
 *    File name   : extcon_test.C
 *    Description : External connectors test
 *
 *    History :
 *    1. Date        : 25, January 2010
 *       Author      : Aleksandar Mitev
 *       Description : Create
 *
 **************************************************************************/
#include <stdio.h>

#include "GenericTypeDefs.h"
#include "Compiler.h"
#include "extcon_test.h"

//static char msg[60];

#define GET_PIN(pin) (1 << (pin))

// definitions for the tested pins
PORTS_NAMES(PortsNames);
EXT_CON_NAMES(ExtConNames);
	
typedef enum _TEST_TYPE {
	   TEST_NONE = 0,
	   TEST_ALL_SET,
	   TEST_ALL_CLEAR,
	   TEST_MOVING_ZERO,
	   TEST_MOVING_ONE,
	
	   TEST_FINISHED				
} TEST_TYPE;
	
static TEST_TYPE testType;

// wait for some time
void WaitSomeTime(void) {
	long i;
	for(i = 0; i < 1000; i++);
}

// make output
void MakePinInput(const ROM PIN* pPin)
{
	BYTE bitMask = GET_PIN(pPin->GPIO_Pin);
	
	switch(pPin->GPIO_Port) {
	case GPIOA:
		TRISA |= bitMask;
		break;
	case GPIOB:
		TRISB |= bitMask;
		break;
	case GPIOC:
		TRISC |= bitMask;
		break;
	}	
	WaitSomeTime();
}

// make input
void MakePinOutput(const ROM PIN* pPin)
{
	BYTE bitMask = ~GET_PIN(pPin->GPIO_Pin);
	
	switch(pPin->GPIO_Port) {
	case GPIOA:
		TRISA &= bitMask;
		break;
	case GPIOB:
		TRISB &= bitMask;
		break;
	case GPIOC:
		TRISC &= bitMask;
		break;
	}	
	WaitSomeTime();
}

// write 1 to output
void SetPin(const ROM PIN *pPin)
{
	BYTE bitMask = GET_PIN(pPin->GPIO_Pin);
	
	switch(pPin->GPIO_Port) {
	case GPIOA:
		LATA |= bitMask;
		break;
	case GPIOB:
		LATB |= bitMask;
		break;
	case GPIOC:
		LATC |= bitMask;
		break;
	}	
}

// write 0 to output
void ClearPin(const ROM PIN *pPin)
{
	BYTE bitMask = ~GET_PIN(pPin->GPIO_Pin);
	
	switch(pPin->GPIO_Port) {
	case GPIOA:
		LATA &= bitMask;
		break;
	case GPIOB:
		LATB &= bitMask;
		break;
	case GPIOC:
		LATC &= bitMask;
		break;
	}	
}

// read input pin
BYTE ReadPin(const ROM PIN *pPin)
{
	BYTE bitMask = GET_PIN(pPin->GPIO_Pin);
	BYTE result = 0;
	
	switch(pPin->GPIO_Port) {
	case GPIOA:
		result = PORTA & bitMask;
		break;
	case GPIOB:
		result = PORTB & bitMask;
		break;
	case GPIOC:
		result = PORTC & bitMask;
		break;
	}	
	
	return (result ? 1 : 0);
}

// Initial configuration of the test
static void ExtConConfigure(ROM EXT_CON_DESC *conn)
{
	int i;
		
	// make main pin output
	MakePinOutput(conn->pullPin);
		
	// make all the other pins inputs
	for(i = 0; i < conn->numOfPins; i++)
		MakePinInput(conn->pins + i);
	
}


// the test
int ExtConTest(ROM EXT_CON_DESC *conn, int startFlag)
{
	static int i, j;
	static int errorFlag, allTestErrFlag;
	
	// initialize if needed
	if(startFlag) {
		errorFlag = 0;
		allTestErrFlag = 0;
		
		ExtConConfigure(conn);
		
		testType = TEST_ALL_SET;
		return 0;
	}	
		
	if(testType != TEST_FINISHED) {
		errorFlag = 0;
		switch(testType) {
		case TEST_ALL_SET:			
			// set the main pin
			SetPin(conn->pullPin);
			
			// wait for some time
			WaitSomeTime();
			
			// check the main pin
			if( !ReadPin(conn->pullPin) ) {
				errorFlag = -1;
			}
			
			// check all the other pins if set
			for(i = 0; i < conn->numOfPins; i++) {
				if( !ReadPin(conn->pins + i) ) {
					errorFlag = -1;
					// report error
				}
			}
			if(!errorFlag) {
				// report test ok
			}
			
			testType++;
			break;
			
		case TEST_ALL_CLEAR:			
			// clear the main pin
			ClearPin(conn->pullPin);
			
			// wait for some time
			WaitSomeTime();
			
			// check the main pin
			if( ReadPin(conn->pullPin) ) {
				errorFlag = -1;
			}
			
			// check all the other pins if clear
			for(i = 0; i < conn->numOfPins; i++) {
				if( ReadPin(conn->pins + i) ) {
					errorFlag = -1;
				}
			}
			if(!errorFlag) {
			}
			
			testType++;		
			break;
			
		case TEST_MOVING_ZERO:
			// set the main pin
			SetPin(conn->pullPin);
						
			// check all the other pins
			for(i = 0; i < conn->numOfPins; i++) {
				// make output and clear a pin
				MakePinOutput(conn->pins + i);
				ClearPin(conn->pins + i);

				// wait for some time
				WaitSomeTime();
					
				for(j = 0; j < conn->numOfPins; j++) {
					// check all the input pins
			   		if( (i != j) && !ReadPin(conn->pins + j) ) {
				   		
//				   		// FIX FOR SAME PIN CONNECTED TO 2 PINS OF THE UEXT
//				   		if( ((i == 2 && j == 3) || (i == 3 && j == 2)) ||
//				   			((i == 1 && j == 5) || (i == 5 && j == 1)) )
//				   			continue;
				   		
						errorFlag = -1;
					}
				}
					
				// check the main pin
				if( !ReadPin(conn->pullPin) ) {
					errorFlag = -1;
				}

				// check the cleared pin
				if( ReadPin(conn->pins + i) ) {
					errorFlag = -1;
				}
				
				// make an input again
				MakePinInput(conn->pins + i);
				
			}
			if(!errorFlag) {
				// report test ok
			}
			testType++;
			break;
			
		case TEST_MOVING_ONE:
			// clear the main pin
			ClearPin(conn->pullPin);
						
			// check all the other pins
			for(i = 0; i < conn->numOfPins; i++) {
				// make output and set a pin
				MakePinOutput(conn->pins + i);
				SetPin(conn->pins + i);

				// wait for some time
				WaitSomeTime();
					
				for(j = 0; j < conn->numOfPins; j++) {
					// check all the input pins
			   		if( (i != j) && ReadPin(conn->pins + j) ) {
				   		
//				   		// FIX FOR SAME PIN CONNECTED TO 2 PINS OF THE UEXT
//				   		if( ((i == 2 && j == 3) || (i == 3 && j == 2)) ||
//				   			((i == 1 && j == 5) || (i == 5 && j == 1)) )
//				   			continue;				   		
				   		
						errorFlag = -1;
					}
				}

				// check the main pin
				if( ReadPin(conn->pullPin) ) {
					errorFlag = -1;
				}

				// check the set pin
				if( !ReadPin(conn->pins + i) ) {
					errorFlag = -1;
				}
				
				
				// make an input again
				MakePinInput(conn->pins + i);
				
			}
			if(!errorFlag) {
				// report test ok

			}

			testType++;
			break;
		}
		
		allTestErrFlag |= errorFlag;
	}
	else {	
		// all tests finished			
		MakePinInput(conn->pullPin);
		
		return (allTestErrFlag) ? -1 : 1;
	}
	
	return 0;	
}

int ExtConConnected(ROM EXT_CON_DESC *conn) 
{
	return 0;
}	
